package uci.inf221.lunarLander.arch;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.UUID;

import uci.inf221.lunarLander.utils.ObjectCloner;

public abstract class Message implements Serializable, Cloneable {

	private static final long serialVersionUID = 5688783805446150462L;

	private transient UUID messageId = UUID.randomUUID();

	@Override
	public Message clone() {
		Message clone = ObjectCloner.deepCopy(this);
		clone.messageId = this.messageId;
		return clone;
	}

	public UUID getMessageId() {
		return messageId;
	}

	@Override
	public String toString() {
		String dataString = "";

		Class<?> clazz = this.getClass();
		while (clazz != Message.class) {
			for (Field field : clazz.getDeclaredFields()) {
				if (!Modifier.isTransient(field.getModifiers())
						&& !Modifier.isStatic(field.getModifiers())) {
					if (dataString.length() > 1)
						dataString += ", ";
					field.setAccessible(true);
					try {
						Object object = field.get(this);
						dataString += field.getName()
								+ ":"
								+ ((object == null) ? object : object
										.toString());
					} catch (IllegalArgumentException e) {
					} catch (IllegalAccessException e) {
					}
				}
			}
			clazz = clazz.getSuperclass();
		}

		return getClass().getSimpleName() + " " + messageId + " (" + dataString
				+ ")";
	}
}
