package uci.inf221.lunarLander.arch.c2;

import java.util.Queue;
import java.util.concurrent.ConcurrentLinkedQueue;

import org.apache.log4j.Logger;

import uci.inf221.lunarLander.arch.Connector;
import uci.inf221.lunarLander.arch.Port;
import uci.inf221.lunarLander.utils.TerminateableLoopThread;

public abstract class C2Connector extends Connector {

	protected Queue<C2Notification> notificationQueue = new ConcurrentLinkedQueue<C2Notification>();
	protected Queue<C2Request> requestQueue = new ConcurrentLinkedQueue<C2Request>();

	private TerminateableLoopThread notificationQueueProcessor;
	private TerminateableLoopThread requestQueueProcessor;

	private final Logger log = Logger.getLogger(this.getClass());

	public C2Connector() {
		super();

		notificationQueueProcessor = new TerminateableLoopThread(
				new Runnable() {
					@Override
					public void run() {
						processNotificationQueue();
					}
				}, 1);

		requestQueueProcessor = new TerminateableLoopThread(new Runnable() {
			@Override
			public void run() {
				processRequestQueue();
			}
		}, 1);

		createPorts();
	}

	protected abstract void createPorts();

	public void start() {
		notificationQueueProcessor.start();
		requestQueueProcessor.start();
	}

	public void stop() {
		notificationQueueProcessor.terminate();
		requestQueueProcessor.terminate();
	}

	public void waitForStop() {
		while (notificationQueueProcessor.isAlive()
				|| requestQueueProcessor.isAlive()) {
			try {
				Thread.sleep(10);
			} catch (InterruptedException e) {
			}
		}
	}

	protected C2TopPort addTopPort(String identifier) {
		C2TopPort port = new TopPort();
		addPort(identifier, port);
		return port;
	}

	protected C2BottomPort addBottomPort(String identifier) {
		C2BottomPort port = new BottomPort();
		addPort(identifier, port);
		return port;
	}

	protected void processNotificationQueue() {
		C2Notification message;
		while ((message = notificationQueue.poll()) != null) {
			log.trace("sending notification: "
					+ message.getClass().getSimpleName());
			for (final Port port : ports.values()) {
				if (port instanceof BottomPort) {
					final C2Notification clonedMessage = (C2Notification) message
							.clone();
					((BottomPort) port).sendNotification(clonedMessage);
				}
			}
		}
	}

	protected void processRequestQueue() {
		C2Request message;
		while ((message = requestQueue.poll()) != null) {
			log.trace("sending request: " + message.getClass().getSimpleName());
			for (final Port port : ports.values()) {
				if (port instanceof TopPort) {
					final C2Request clonedMessage = (C2Request) message.clone();
					((TopPort) port).sendRequest(clonedMessage);
				}
			}
		}
	}

	private class TopPort extends C2TopPort {
		@Override
		protected void receiveNotification(C2Notification notification) {
			log.trace("receiving notification: "
					+ notification.getClass().getSimpleName());
			notificationQueue.add(notification);
		}
	}

	private class BottomPort extends C2BottomPort {
		@Override
		protected void receiveRequest(C2Request request) {
			log.trace("receiving request: "
					+ request.getClass().getSimpleName());
			requestQueue.add(request);
		}
	}
}
