package uci.inf221.lunarLander.dataStore.entities;

import java.io.Serializable;

public class Level extends DataEntity {
	public enum LevelStatus implements Serializable {
		INITIALISING, LOADED, READY, STARTED, COMPLETED, FAILED, ERROR;

		public boolean hasEnded() {
			return this == ERROR || this == FAILED || this == COMPLETED;
		}
	}

	private static final int CHARACTER_EXTENT = 8;

	private static final long serialVersionUID = 5665811371421697022L;

	private String textRepresentation;
	private transient boolean[][] map;

	private LevelStatus levelStatus;

	public String getTextRepresentation() {
		return textRepresentation;
	}

	public void setTextRepresentation(String textRepresentation) {
		this.textRepresentation = textRepresentation;
	}

	public boolean[][] getMap() {
		if (map == null) {
			loadMap();
		}
		return map;
	}

	private void loadMap() {
		if (textRepresentation == null) {
			throw new RuntimeException("level file empty");
		}

		String[] split = textRepresentation.trim().split("\n");
		if (split.length > 0) {
			map = new boolean[split[0].trim().length() * CHARACTER_EXTENT][split.length
					* CHARACTER_EXTENT];
			int y = 0;
			int x = 0;
			for (String line : split) {
				line = line.trim();
				if (!line.isEmpty()) {
					for (char character : line.toCharArray()) {
						if (character == '.') {
							for (int xx = 0; xx < CHARACTER_EXTENT; xx++) {
								for (int yy = 0; yy < CHARACTER_EXTENT; yy++) {
									map[x + xx][y + yy] = false;
								}
							}
						} else if (character == 'x') {
							for (int xx = 0; xx < CHARACTER_EXTENT; xx++) {
								for (int yy = 0; yy < CHARACTER_EXTENT; yy++) {
									map[x + xx][y + yy] = true;
								}
							}
						} else {
							throw new RuntimeException(
									"illegal character in level file: "
											+ Character.toString(character));
						}
						x += CHARACTER_EXTENT;
					}
					x = 0;
					y += CHARACTER_EXTENT;
				}
			}
		} else {
			throw new RuntimeException("level file empty");
		}
	}

	public LevelStatus getLevelStatus() {
		return levelStatus;
	}

	public void setLevelStatus(LevelStatus levelStatus) {
		this.levelStatus = levelStatus;
	}

	public int getWidth() {
		return getMap().length;
	}

	public int getHeight() {
		if (getWidth() > 0) {
			return getMap()[0].length;
		}
		return 0;
	}

}
