package uci.inf221.lunarLander.dataStore.entities;

import java.io.Serializable;

public class MovementVector implements Serializable {

	private static final long serialVersionUID = -1056348555964961069L;

	// TODO adjust constants ?

	// pixels per second² (vector)
	private static final float[] GRAVITY = new float[] { 0, 0.003f / 1000 };

	private float[] position = new float[2];
	private float[] speed = new float[2];
	private float[] acceleration = new float[2];
	private long clockTick;
	private boolean gravity;

	public float[] getPosition() {
		return position;
	}

	public void setPosition(float[] position) {
		this.position = position;
	}

	public float[] getSpeedVector() {
		float[] speed = new float[2];
		speed[0] = (float) (this.speed[1] * Math.cos(this.speed[0]));
		speed[1] = (float) (this.speed[1] * Math.sin(this.speed[0]));
		return speed;
	}

	public void setSpeedVector(float[] speed) {
		this.speed[0] = (float) Math.atan2(speed[1], speed[0]);
		this.speed[1] = (float) Math.sqrt(speed[1] * speed[1] + speed[0]
				* speed[0]);
	}

	public void setSpeed(float dir, float mag) {
		speed[0] = normalizeDir(dir);
		speed[1] = mag;
	}

	public float[] getAccelerationVector() {
		float[] acceleration = new float[2];
		acceleration[0] = (float) (this.acceleration[1] * Math
				.cos(this.acceleration[0]));
		acceleration[1] = (float) (this.acceleration[1] * Math
				.sin(this.acceleration[0]));
		return acceleration;
	}

	public void setAccelerationVector(float[] acceleration) {
		this.acceleration[0] = (float) Math.atan2(acceleration[1],
				acceleration[0]);
		this.acceleration[1] = (float) Math.sqrt(acceleration[1]
				* acceleration[1] + acceleration[0] * acceleration[0]);
	}

	public void setAcceleration(float dir, float mag) {
		acceleration[0] = normalizeDir(dir);
		acceleration[1] = mag;
	}

	public boolean isGravity() {
		return gravity;
	}

	public void setGravity(boolean gravity) {
		this.gravity = gravity;
	}

	public long getClockTick() {
		return clockTick;
	}

	public void setClockTick(long clockTick) {
		this.clockTick = clockTick;
	}

	public MovementVector moveToTime(long newClockTick) {
		MovementVector vector = new MovementVector();
		vector.setClockTick(newClockTick);
		vector.setAcceleration(acceleration[0], acceleration[1]);
		vector.setGravity(gravity);

		int tickDiff = (int) (newClockTick - clockTick);
		if (clockTick == 0)
			tickDiff = 0;

		float[] acceleration = getAccelerationVector();
		if (gravity) {
			acceleration[0] += GRAVITY[0];
			acceleration[1] += GRAVITY[1];
		}

		float[] speed = getSpeedVector();

		float[] newPosition = new float[] {
				position[0] + speed[0] * tickDiff + acceleration[0] * (int) 0.5
						* tickDiff * tickDiff,
				position[1] + speed[1] * tickDiff + acceleration[1] * (int) 0.5
						* tickDiff * tickDiff };
		float[] newSpeed = new float[] { speed[0] + acceleration[0] * tickDiff,
				speed[1] + acceleration[1] * tickDiff };

		vector.setPosition(newPosition);
		vector.setSpeedVector(newSpeed);
		return vector;
	}

	public float getAccelerationDirection() {
		return acceleration[0];
	}

	public float getAccelerationMagnitude() {
		return acceleration[1];
	}

	public float getSpeedDirection() {
		return speed[0];
	}

	public float getSpeedMagnitude() {
		return speed[1];
	}

	public void turnAcceleration(float turnAngle) {
		setAcceleration(getAccelerationDirection() + turnAngle,
				getAccelerationMagnitude());
	}

	public void addAcceleration(float incAcceleration) {
		setAcceleration(getAccelerationDirection(), getAccelerationMagnitude()
				+ incAcceleration);
	}

	public float[] getSpeed() {
		return speed;
	}

	public void setSpeed(float[] speed) {
		speed[0] = normalizeDir(speed[0]);
		this.speed = speed;
	}

	public float[] getAcceleration() {
		return acceleration;
	}

	public void setAcceleration(float[] acceleration) {
		acceleration[0] = normalizeDir(acceleration[0]);
		this.acceleration = acceleration;
	}

	private float normalizeDir(float dir) {
		while (dir < -Math.PI)
			dir += 2 * Math.PI;
		while (dir > Math.PI)
			dir -= 2 * Math.PI;
		return dir;
	}

}
