package uci.inf221.lunarLander.gui.impl;

import java.awt.GraphicsConfiguration;
import java.awt.GraphicsEnvironment;
import java.awt.Image;
import java.awt.Transparency;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.imageio.ImageIO;

import org.apache.log4j.Logger;

public class SpriteStore {

	public static final String SPRITE_ASTEROID = "sprites/asteroid.png";
	public static final String SPRITE_BULLET = "sprites/bullet.png";
	public static final String SPRITE_SHIP = "sprites/ship.png";
	public static final String[] SPRITE_THRUST = new String[] {
			"sprites/thrust_0.png", "sprites/thrust_1.png",
			"sprites/thrust_2.png", "sprites/thrust_3.png",
			"sprites/thrust_4.png", "sprites/thrust_5.png",
			"sprites/thrust_6.png", "sprites/thrust_7.png",
			"sprites/thrust_8.png", "sprites/thrust_9.png", };

	private static SpriteStore instance = new SpriteStore();

	private final Logger log = Logger.getLogger(this.getClass());

	public static SpriteStore getInstance() {
		return instance;
	}

	private Map<String, Sprite> sprites = new HashMap<String, Sprite>();

	public SpriteStore() {
		// preload sprites
		getSprite(SPRITE_ASTEROID);
		getSprite(SPRITE_BULLET);
		getSprite(SPRITE_SHIP);

		for (String ref : SPRITE_THRUST) {
			getSprite(ref);
		}
	}

	public Sprite getSprite(String ref) {
		Sprite sprite = sprites.get(ref);
		if (sprite != null) {
			return sprite;
		}

		BufferedImage sourceImage = null;

		try {
			File file = new File(ref);

			if (!file.exists()) {
				log.error("sprite file missing: " + ref);
				return null;
			}

			sourceImage = ImageIO.read(file);
		} catch (IOException e) {
			log.error("failed to load sprite: " + ref);
			return null;
		}

		// create an accelerated image of the right size to store our sprite in

		GraphicsConfiguration gc = GraphicsEnvironment
				.getLocalGraphicsEnvironment().getDefaultScreenDevice()
				.getDefaultConfiguration();
		Image image = gc.createCompatibleImage(sourceImage.getWidth(),
				sourceImage.getHeight(), Transparency.TRANSLUCENT);

		// draw our source image into the accelerated image

		image.getGraphics().drawImage(sourceImage, 0, 0, null);

		sprite = new Sprite(image);
		sprites.put(ref, sprite);

		return sprite;
	}
}
