package uci.inf221.lunarLander.net.connector;

import uci.inf221.lunarLander.arch.c2.C2Connector;
import uci.inf221.lunarLander.net.connector.serverToClients.component.ClientConnectionManagerComponent;
import uci.inf221.lunarLander.net.connector.serverToClients.component.ClientConnectionStateChangeHandlerComponent;
import uci.inf221.lunarLander.net.connector.serverToClients.component.ClientMessageHandlerComponent;
import uci.inf221.lunarLander.net.connector.serverToClients.component.ClientStatesComponent;
import uci.inf221.lunarLander.net.connector.serverToClients.component.ServerMessageComposerComponent;
import uci.inf221.lunarLander.net.connector.serverToClients.connector.ConnectionManagerConnector;
import uci.inf221.lunarLander.net.connector.serverToClients.connector.DataStorePortConnector;

public class ServerToClientsConnector extends C2Connector {

	public static final String PORT_IDENTIFIER_DATA_STORE_CONNECTOR = "data_store";

	private ClientStatesComponent clientStatesComponent;
	private ServerMessageComposerComponent serverMessageComposerComponent;
	private ClientMessageHandlerComponent clientMessageHandlerComponent;
	private ClientConnectionStateChangeHandlerComponent clientConnectionStateChangeHandlerComponent;
	private ClientConnectionManagerComponent clientConnectionManagerComponent;

	private DataStorePortConnector dataStorePortConnector;
	private ConnectionManagerConnector connectionManagerConnector;

	@Override
	protected void createPorts() {
		createInnerStructure();

		addPort(PORT_IDENTIFIER_DATA_STORE_CONNECTOR,
				dataStorePortConnector
						.getPort(DataStorePortConnector.PORT_IDENTIFIER_SERVER_TO_CLIENTS_CONNECTOR));
	}

	private void createInnerStructure() {
		clientStatesComponent = new ClientStatesComponent();
		serverMessageComposerComponent = new ServerMessageComposerComponent();
		clientMessageHandlerComponent = new ClientMessageHandlerComponent();
		clientConnectionStateChangeHandlerComponent = new ClientConnectionStateChangeHandlerComponent();
		clientConnectionManagerComponent = new ClientConnectionManagerComponent();

		dataStorePortConnector = new DataStorePortConnector();
		connectionManagerConnector = new ConnectionManagerConnector();

		dataStorePortConnector
				.getPort(
						DataStorePortConnector.PORT_IDENTIFIER_CLIENT_STATES_COMPONENT)
				.connectTo(
						clientStatesComponent
								.getPort(ClientStatesComponent.PORT_IDENTIFIER_DATA_STORE_PORT_CONNECTOR));
		dataStorePortConnector
				.getPort(
						DataStorePortConnector.PORT_IDENTIFIER_SERVER_MESSAGE_COMPOSER_COMPONENT)
				.connectTo(
						serverMessageComposerComponent
								.getPort(ServerMessageComposerComponent.PORT_IDENTIFIER_DATA_STORE_PORT_CONNECTOR));
		dataStorePortConnector
				.getPort(
						DataStorePortConnector.PORT_IDENTIFIER_CLIENT_MESSAGE_HANDLER_COMPONENT)
				.connectTo(
						clientMessageHandlerComponent
								.getPort(ClientMessageHandlerComponent.PORT_IDENTIFIER_DATA_STORE_PORT_CONNECTOR));
		dataStorePortConnector
				.getPort(
						DataStorePortConnector.PORT_IDENTIFIER_CLIENT_CONNECTION_STATE_CHANGE_HANDLER_COMPONENT)
				.connectTo(
						clientConnectionStateChangeHandlerComponent
								.getPort(ClientConnectionStateChangeHandlerComponent.PORT_IDENTIFIER_DATA_STORE_PORT_CONNECTOR));

		connectionManagerConnector
				.getPort(
						ConnectionManagerConnector.PORT_IDENTIFIER_SERVER_MESSAGE_COMPOSER_COMPONENT)
				.connectTo(
						serverMessageComposerComponent
								.getPort(ServerMessageComposerComponent.PORT_IDENTIFIER_CONNECTION_MANAGER_CONNECTOR));
		connectionManagerConnector
				.getPort(
						ConnectionManagerConnector.PORT_IDENTIFIER_CLIENT_MESSAGE_HANDLER_COMPONENT)
				.connectTo(
						clientMessageHandlerComponent
								.getPort(ClientMessageHandlerComponent.PORT_IDENTIFIER_CONNECTION_MANAGER_CONNECTOR));
		connectionManagerConnector
				.getPort(
						ConnectionManagerConnector.PORT_IDENTIFIER_CLIENT_CONNECTION_STATE_CHANGE_HANDLER_COMPONENT)
				.connectTo(
						clientConnectionStateChangeHandlerComponent
								.getPort(ClientConnectionStateChangeHandlerComponent.PORT_IDENTIFIER_CONNECTION_MANAGER_CONNECTOR));
		connectionManagerConnector
				.getPort(
						ConnectionManagerConnector.PORT_IDENTIFIER_CLIENT_CONNECTION_MANAGER_COMPONENT)
				.connectTo(
						clientConnectionManagerComponent
								.getPort(ClientConnectionManagerComponent.PORT_IDENTIFIER_CONNECTION_MANAGER_CONNECTOR));
	}

	@Override
	public void start() {
		clientStatesComponent.start();
		serverMessageComposerComponent.start();
		clientMessageHandlerComponent.start();
		clientConnectionStateChangeHandlerComponent.start();
		clientConnectionManagerComponent.start();

		dataStorePortConnector.start();
		connectionManagerConnector.start();
	}

	@Override
	public void stop() {
		clientConnectionManagerComponent.stop();
		clientConnectionStateChangeHandlerComponent.stop();
		clientMessageHandlerComponent.stop();
		serverMessageComposerComponent.stop();
		clientStatesComponent.stop();

		connectionManagerConnector.stop();
		dataStorePortConnector.stop();
	}

}
