package uci.inf221.lunarLander.net.connector.clientToServer.component;

import org.apache.log4j.Logger;

import uci.inf221.lunarLander.arch.Message;
import uci.inf221.lunarLander.arch.Port.PortDirection;
import uci.inf221.lunarLander.arch.c2.C2Component;
import uci.inf221.lunarLander.arch.c2.C2Notification;
import uci.inf221.lunarLander.arch.c2.C2Request;
import uci.inf221.lunarLander.dataStore.entities.Asteroid;
import uci.inf221.lunarLander.dataStore.entities.Bullet;
import uci.inf221.lunarLander.dataStore.entities.DataEntity;
import uci.inf221.lunarLander.dataStore.entities.Level;
import uci.inf221.lunarLander.dataStore.entities.Ship;
import uci.inf221.lunarLander.dataStore.message.CreateNewAsteroidRequest;
import uci.inf221.lunarLander.dataStore.message.CreateNewBulletRequest;
import uci.inf221.lunarLander.dataStore.message.CreateNewShipRequest;
import uci.inf221.lunarLander.dataStore.message.CurrentEntityDataNotification;
import uci.inf221.lunarLander.dataStore.message.CurrentGameStartTimeNotification;
import uci.inf221.lunarLander.dataStore.message.DestroyEntityRequest;
import uci.inf221.lunarLander.dataStore.message.EntityDestroyedNotification;
import uci.inf221.lunarLander.dataStore.message.GameStartTimeUpdatedNotification;
import uci.inf221.lunarLander.dataStore.message.LevelStatusUpdatedNotification;
import uci.inf221.lunarLander.dataStore.message.LevelUpdatedNotification;
import uci.inf221.lunarLander.dataStore.message.MovementVectorUpdatedNotification;
import uci.inf221.lunarLander.dataStore.message.NewAsteroidNotification;
import uci.inf221.lunarLander.dataStore.message.NewBulletNotification;
import uci.inf221.lunarLander.dataStore.message.NewShipNotification;
import uci.inf221.lunarLander.dataStore.message.UpdateGameStartTimeRequest;
import uci.inf221.lunarLander.dataStore.message.UpdateLevelMapRequest;
import uci.inf221.lunarLander.dataStore.message.UpdateLevelStatusRequest;
import uci.inf221.lunarLander.dataStore.message.UpdateMovementVectorRequest;
import uci.inf221.lunarLander.net.connector.clientToServer.message.HandleIncomingMessageRequest;
import uci.inf221.lunarLander.net.connector.serverToClients.entity.RemoteConnectionId;

public class ServerMessageHandlerComponent extends C2Component {

	public static final String PORT_IDENTIFIER_DATA_STORE_PORT_CONNECTOR = "data_store_port";
	public static final String PORT_IDENTIFIER_CONNECTION_MANAGER_CONNECTOR = "connection_manager";

	private final Logger log = Logger.getLogger(this.getClass());

	@Override
	protected void createPorts() {
		addTopPort(PORT_IDENTIFIER_DATA_STORE_PORT_CONNECTOR).setDirection(
				PortDirection.OUT);
		addBottomPort(PORT_IDENTIFIER_CONNECTION_MANAGER_CONNECTOR)
				.setDirection(PortDirection.IN);
	}

	@Override
	protected void initialize() {
	}

	@Override
	public void start() {
	}

	@Override
	public void stop() {
	}

	@Override
	public void waitForStop() {
	}

	@Override
	protected void receiveNotification(C2Notification notification) {
	}

	@Override
	protected void receiveRequest(C2Request request) {
		if (request instanceof HandleIncomingMessageRequest) {
			Message message = ((HandleIncomingMessageRequest) request)
					.getMessage();

			if (message instanceof EntityDestroyedNotification) {
				sendRequest(new DestroyEntityRequest(this,
						new RemoteConnectionId(-1),
						((EntityDestroyedNotification) message).getEntityId()));
			} else if (message instanceof LevelStatusUpdatedNotification) {
				sendRequest(new UpdateLevelStatusRequest(this,
						new RemoteConnectionId(-1),
						((LevelStatusUpdatedNotification) message)
								.getLevelStatus()));
			} else if (message instanceof LevelUpdatedNotification) {
				sendRequest(new UpdateLevelMapRequest(this,
						new RemoteConnectionId(-1),
						((LevelUpdatedNotification) message).getLevel()
								.getTextRepresentation()));
				sendRequest(new UpdateLevelStatusRequest(this,
						new RemoteConnectionId(-1),
						((LevelUpdatedNotification) message).getLevel()
								.getLevelStatus()));
			} else if (message instanceof MovementVectorUpdatedNotification) {
				sendRequest(new UpdateMovementVectorRequest(this,
						new RemoteConnectionId(-1),
						((MovementVectorUpdatedNotification) message)
								.getEntityId(),
						((MovementVectorUpdatedNotification) message)
								.getMovementVector()));
			} else if (message instanceof NewAsteroidNotification) {
				sendRequest(new CreateNewAsteroidRequest(this,
						new RemoteConnectionId(-1),
						((NewAsteroidNotification) message).getAsteroid()));
			} else if (message instanceof NewBulletNotification) {
				sendRequest(new CreateNewBulletRequest(this,
						new RemoteConnectionId(-1),
						((NewBulletNotification) message).getBullet()));
			} else if (message instanceof NewShipNotification) {
				sendRequest(new CreateNewShipRequest(this,
						new RemoteConnectionId(-1),
						((NewShipNotification) message).getShip()));
			} else if (message instanceof CurrentGameStartTimeNotification) {
				sendRequest(new UpdateGameStartTimeRequest(this,
						new RemoteConnectionId(-1),
						((CurrentGameStartTimeNotification) message)
								.getStartTime()));
			} else if (message instanceof GameStartTimeUpdatedNotification) {
				sendRequest(new UpdateGameStartTimeRequest(this,
						new RemoteConnectionId(-1),
						((GameStartTimeUpdatedNotification) message)
								.getStartTime()));
			} else if (message instanceof CurrentEntityDataNotification) {
				for (DataEntity entity : ((CurrentEntityDataNotification) message)
						.getEntities()) {
					if (entity instanceof Level) {
						sendRequest(new UpdateLevelMapRequest(this,
								new RemoteConnectionId(-1),
								((Level) entity).getTextRepresentation()));
						sendRequest(new UpdateLevelStatusRequest(this,
								new RemoteConnectionId(-1),
								((Level) entity).getLevelStatus()));
					} else if (entity instanceof Asteroid) {
						sendRequest(new CreateNewAsteroidRequest(this,
								new RemoteConnectionId(-1), (Asteroid) entity));
					} else if (entity instanceof Bullet) {
						sendRequest(new CreateNewBulletRequest(this,
								new RemoteConnectionId(-1), (Bullet) entity));
					} else if (entity instanceof Ship) {
						sendRequest(new CreateNewShipRequest(this,
								new RemoteConnectionId(-1), (Ship) entity));
					}
				}
			} else {
				log.warn("ignoring server message: " + message.toString());
			}
		}
	}

}
