package uci.inf221.lunarLander.net.connector.serverToClients.component;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

import uci.inf221.lunarLander.arch.Port.PortDirection;
import uci.inf221.lunarLander.arch.c2.C2Component;
import uci.inf221.lunarLander.arch.c2.C2Notification;
import uci.inf221.lunarLander.arch.c2.C2Request;
import uci.inf221.lunarLander.dataStore.message.ChangeNTPServerRequest;
import uci.inf221.lunarLander.dataStore.message.DestroyEntityRequest;
import uci.inf221.lunarLander.dataStore.message.GetCurrentDataRequest;
import uci.inf221.lunarLander.net.connector.serverToClients.entity.ClientState;
import uci.inf221.lunarLander.net.connector.serverToClients.entity.RemoteConnectionId;
import uci.inf221.lunarLander.net.connector.serverToClients.message.ClientStateUpdatedNotification;
import uci.inf221.lunarLander.net.connector.serverToClients.message.HandleConnectionStateChangeRequest;
import uci.inf221.lunarLander.net.connector.serverToClients.message.HandleServerStartedRequest;
import uci.inf221.lunarLander.net.connector.serverToClients.message.UpdateClientConnectionStateRequest;
import uci.inf221.lunarLander.server.Server;

public class ClientConnectionStateChangeHandlerComponent extends C2Component {

	public static final String PORT_IDENTIFIER_DATA_STORE_PORT_CONNECTOR = "data_store_port";
	public static final String PORT_IDENTIFIER_CONNECTION_MANAGER_CONNECTOR = "connection_manager";

	private Map<Integer, UUID> clientShipIds = new HashMap<Integer, UUID>();

	@Override
	protected void createPorts() {
		addTopPort(PORT_IDENTIFIER_DATA_STORE_PORT_CONNECTOR);
		addBottomPort(PORT_IDENTIFIER_CONNECTION_MANAGER_CONNECTOR)
				.setDirection(PortDirection.IN);
	}

	@Override
	protected void initialize() {
	}

	@Override
	public void start() {
	}

	@Override
	public void stop() {
	}

	@Override
	public void waitForStop() {
	}

	@Override
	protected void receiveNotification(C2Notification notification) {
		if (notification instanceof ClientStateUpdatedNotification) {
			ClientState state = ((ClientStateUpdatedNotification) notification)
					.getState();
			UUID shipId = state.getShipId();
			if (shipId != null) {
				clientShipIds.put(state.getRemoteConnectionId(), shipId);
			}
		}
	}

	@Override
	protected void receiveRequest(C2Request request) {
		if (request instanceof HandleConnectionStateChangeRequest) {
			int remoteConnectionId = ((HandleConnectionStateChangeRequest) request)
					.getRemoteConnectionId();
			boolean connected = ((HandleConnectionStateChangeRequest) request)
					.isConnected();
			sendRequest(new UpdateClientConnectionStateRequest(this,
					remoteConnectionId, connected));

			if (connected) {
				sendRequest(new GetCurrentDataRequest(this,
						new RemoteConnectionId(remoteConnectionId)));
			} else {
				UUID shipId = clientShipIds.get(remoteConnectionId);
				if (shipId != null) {
					sendRequest(new DestroyEntityRequest(this, shipId));
				}
			}
		} else if (request instanceof HandleServerStartedRequest) {
			sendRequest(new ChangeNTPServerRequest(this,
					((HandleServerStartedRequest) request).getLocalAddress(),
					Server.NTP_PORT));
		}
	}

}
