package uci.inf221.lunarLander.net.connector.serverToClients.component;

import org.apache.log4j.Logger;

import uci.inf221.lunarLander.arch.Message;
import uci.inf221.lunarLander.arch.Port.PortDirection;
import uci.inf221.lunarLander.arch.c2.C2Component;
import uci.inf221.lunarLander.arch.c2.C2Notification;
import uci.inf221.lunarLander.arch.c2.C2Request;
import uci.inf221.lunarLander.dataStore.message.CreateNewBulletRequest;
import uci.inf221.lunarLander.dataStore.message.CreateNewShipRequest;
import uci.inf221.lunarLander.dataStore.message.DestroyAllEntitiesRequest;
import uci.inf221.lunarLander.dataStore.message.DestroyEntityRequest;
import uci.inf221.lunarLander.dataStore.message.EntityDestroyedNotification;
import uci.inf221.lunarLander.dataStore.message.GameStartTimeUpdatedNotification;
import uci.inf221.lunarLander.dataStore.message.LevelStatusUpdatedNotification;
import uci.inf221.lunarLander.dataStore.message.MovementVectorUpdatedNotification;
import uci.inf221.lunarLander.dataStore.message.NewBulletNotification;
import uci.inf221.lunarLander.dataStore.message.NewShipNotification;
import uci.inf221.lunarLander.dataStore.message.UpdateGameStartTimeRequest;
import uci.inf221.lunarLander.dataStore.message.UpdateLevelStatusRequest;
import uci.inf221.lunarLander.dataStore.message.UpdateMovementVectorRequest;
import uci.inf221.lunarLander.net.connector.serverToClients.entity.RemoteConnectionId;
import uci.inf221.lunarLander.net.connector.serverToClients.message.HandleIncomingMessageRequest;
import uci.inf221.lunarLander.net.connector.serverToClients.message.UpdateClientIdRequest;

public class ClientMessageHandlerComponent extends C2Component {

	public static final String PORT_IDENTIFIER_DATA_STORE_PORT_CONNECTOR = "data_store_port";
	public static final String PORT_IDENTIFIER_CONNECTION_MANAGER_CONNECTOR = "connection_manager";

	private final Logger log = Logger.getLogger(this.getClass());

	@Override
	protected void createPorts() {
		addTopPort(PORT_IDENTIFIER_DATA_STORE_PORT_CONNECTOR).setDirection(
				PortDirection.OUT);
		addBottomPort(PORT_IDENTIFIER_CONNECTION_MANAGER_CONNECTOR)
				.setDirection(PortDirection.IN);
	}

	@Override
	protected void initialize() {
	}

	@Override
	public void start() {
	}

	@Override
	public void stop() {
	}

	@Override
	public void waitForStop() {
	}

	@Override
	protected void receiveNotification(C2Notification notification) {
	}

	@Override
	protected void receiveRequest(C2Request request) {
		if (request instanceof HandleIncomingMessageRequest) {
			Message message = ((HandleIncomingMessageRequest) request)
					.getMessage();

			if (message instanceof EntityDestroyedNotification) {
				sendRequest(new DestroyEntityRequest(this,
						new RemoteConnectionId(
								((HandleIncomingMessageRequest) request)
										.getRemoteConnectionId()),
						((EntityDestroyedNotification) message).getEntityId()));
			} else if (message instanceof LevelStatusUpdatedNotification) {
				sendRequest(new UpdateLevelStatusRequest(this,
						new RemoteConnectionId(
								((HandleIncomingMessageRequest) request)
										.getRemoteConnectionId()),
						((LevelStatusUpdatedNotification) message)
								.getLevelStatus()));
				if (((LevelStatusUpdatedNotification) message).getLevelStatus()
						.hasEnded()) {
					sendRequest(new UpdateGameStartTimeRequest(this, 0));
					sendRequest(new DestroyAllEntitiesRequest(this));
				}
			} else if (message instanceof MovementVectorUpdatedNotification) {
				sendRequest(new UpdateMovementVectorRequest(this,
						new RemoteConnectionId(
								((HandleIncomingMessageRequest) request)
										.getRemoteConnectionId()),
						((MovementVectorUpdatedNotification) message)
								.getEntityId(),
						((MovementVectorUpdatedNotification) message)
								.getMovementVector()));
			} else if (message instanceof NewBulletNotification) {
				sendRequest(new CreateNewBulletRequest(this,
						new RemoteConnectionId(
								((HandleIncomingMessageRequest) request)
										.getRemoteConnectionId()),
						((NewBulletNotification) message).getBullet()));
			} else if (message instanceof NewShipNotification) {
				sendRequest(new UpdateClientIdRequest(
						this,
						((HandleIncomingMessageRequest) request)
								.getRemoteConnectionId(),
						((NewShipNotification) message).getShip().getPlayerId(),
						((NewShipNotification) message).getShip().getId()));
				sendRequest(new CreateNewShipRequest(this,
						new RemoteConnectionId(
								((HandleIncomingMessageRequest) request)
										.getRemoteConnectionId()),
						((NewShipNotification) message).getShip()));
			} else if (message instanceof GameStartTimeUpdatedNotification) {
				sendRequest(new UpdateGameStartTimeRequest(this,
						new RemoteConnectionId(
								((HandleIncomingMessageRequest) request)
										.getRemoteConnectionId()),
						((GameStartTimeUpdatedNotification) message)
								.getStartTime()));
			} else {
				log.warn("ignoring client message: " + message.toString());
			}
		}
	}
}
