package uci.inf221.lunarLander.net.connector.serverToClients.component;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

import uci.inf221.lunarLander.arch.c2.C2Component;
import uci.inf221.lunarLander.arch.c2.C2Notification;
import uci.inf221.lunarLander.arch.c2.C2Request;
import uci.inf221.lunarLander.net.connector.serverToClients.entity.ClientState;
import uci.inf221.lunarLander.net.connector.serverToClients.message.ClientStateUpdatedNotification;
import uci.inf221.lunarLander.net.connector.serverToClients.message.UpdateClientConnectionStateRequest;
import uci.inf221.lunarLander.net.connector.serverToClients.message.UpdateClientIdRequest;

public class ClientStatesComponent extends C2Component {

	public static final String PORT_IDENTIFIER_DATA_STORE_PORT_CONNECTOR = "data_store";

	private Map<Integer, ClientState> clientStates = new HashMap<Integer, ClientState>();

	@Override
	protected void createPorts() {
		addBottomPort(PORT_IDENTIFIER_DATA_STORE_PORT_CONNECTOR);
	}

	@Override
	protected void initialize() {
	}

	@Override
	public void start() {
	}

	@Override
	public void stop() {
	}

	@Override
	public void waitForStop() {
	}

	@Override
	protected void receiveNotification(C2Notification notification) {
	}

	@Override
	protected void receiveRequest(C2Request request) {
		if (request instanceof UpdateClientConnectionStateRequest) {
			int remoteConnectionId = ((UpdateClientConnectionStateRequest) request)
					.getRemoteConnectionId();
			boolean connected = ((UpdateClientConnectionStateRequest) request)
					.isConnected();
			ClientState clientState = clientStates.get(remoteConnectionId);
			if (clientState == null) {
				clientState = new ClientState();
				clientState.setRemoteConnectionId(remoteConnectionId);
			}
			clientState.setConnected(connected);
			sendNotification(new ClientStateUpdatedNotification(this,
					clientState));
		} else if (request instanceof UpdateClientIdRequest) {
			int remoteConnectionId = ((UpdateClientIdRequest) request)
					.getRemoteConnectionId();
			UUID playerId = ((UpdateClientIdRequest) request).getPlayerId();
			UUID shipId = ((UpdateClientIdRequest) request).getShipId();
			ClientState clientState = clientStates.get(remoteConnectionId);
			if (clientState == null) {
				clientState = new ClientState();
				clientState.setRemoteConnectionId(remoteConnectionId);
			}
			clientState.setPlayerId(playerId);
			clientState.setShipId(shipId);
			sendNotification(new ClientStateUpdatedNotification(this,
					clientState));
		}
	}

}
